# Notification send message to sources setting in configuration

# usage:
#   WatchDoge::Notification.push(message)

# spec:
#  1. could adding custom sources
#  2. custom source MUST under WatchDoge::Notification namespace and having #push method

# for example:
# class WatchDoge::Notification::CustomSource
#   # args passing from value of WatchDoge::Configuration.notifications['CustomSource']
#   def initialize **args
#     # implement your source setup
#   end
#
#   # called by WatchDoge::Notification.push(msg)
#   def push msg
#     # do your source push here
#   end
# end

# built-in sources
require 'watchdoge/notification/base'

require 'watchdoge/notification/slack_webhook'
require 'watchdoge/notification/mattermost'
require 'watchdoge/notification/gitlab_repo'

module WatchDoge
  class RegressionError < StandardError; end

  module Notification
    class << self
      def push message
        sources do |klass|
          puts "[notification] new msg push to #{klass}"
          klass.push message
        end
      end

      def flush
        raise_err = false

        sources do |klass|
          puts '[notification] flush'
          raise_err = true if klass.msg_size > 0
          klass.flush
        end

        if raise_err
          raise WatchDoge::RegressionError, 'Has detected style changed. Regression failed'
        end
      end

      private

      def sources
        @sources ||= WatchDoge.configuration.notifications

        @sources = {
          base: {}
        } if @sources.empty?

        @sources.each do |klass_sym, source_args|
          sources_map[klass_sym] ||=
            get_source_class(klass_sym).new source_args

          yield(sources_map[klass_sym])
        end
      end

      def sources_map
        @sources_map ||= {}
        @sources_map
      end

      def get_source_class klass_sym
        klass = klass_sym.to_s.split('_').collect(&:capitalize).join

        "WatchDoge::Notification::#{klass}".constantize
      end
    end
  end
end
