# Regression testing realtime screenshot with reference images for scenario described by DSL

# usage:
#   WatchDoge::Regression.run_all_scenarios(:capture)
#   WatchDoge::Regression.run_all_scenarios(:test)

# spec:
#   1. should have rake tasks `rake watchdoge:capture` and `rake watchdoge:test` while in Rails
#   2. able to generate scenario template by Rails generator

# todo: support run single scenario

require 'watchdoge/regression/dsl'
require 'watchdoge/regression/utils'

module WatchDoge
  module Regression
    class Manager
      include WatchDoge::Regression::DSL

      class << self
        def run_all_scenarios regression_flag
          WatchDoge.hooks.before_regression.each { |t| instance_exec &t }

          Dir["#{WatchDoge::Regression::Utils.scenarios_dir}/*"].each do |scenario_path|
            mgr = self.new scenario_path, regression_flag: regression_flag

            WatchDoge.hooks.before_scenario.each { |t| mgr.instance_exec &t }

            if WatchDoge.hooks.around_scenario
              WatchDoge.hooks.around_scenario.call(Proc.new { mgr.eval_scenario })
            else
              mgr.eval_scenario
            end

            WatchDoge.hooks.after_scenario.each { |t| mgr.instance_exec &t }
          end

          WatchDoge.hooks.after_regression.each { |t| instance_exec &t }
          WatchDoge::Notification.flush
        end
      end

      def initialize scenario_path, regression_flag:
        @scenario_path = scenario_path
        @scenario_name = File.basename @scenario_path, '.rb'
        @regression_flag = regression_flag

        @setup_proc = nil
        @teardown_proc = nil

        worker_options =
          WatchDoge.configuration.default_worker_options.merge WatchDoge.configuration.worker_options

        @worker = WatchDoge::Worker.new @scenario_name, worker_options.dup

        @view_port = {}
      end

      def eval_scenario
        puts "scenario: #{@scenario_name} begin"
        instance_eval File.read(@scenario_path), @scenario_path
        @worker.close
        puts "scenario: #{@scenario_name} end\n"
        puts ""
      end

      def reference_image
        WatchDoge::Regression::Utils.reference_dir + "/#{@scenario_name}/#{@reference_file_name}.png"
      end
    end
  end
end
