# WebdriverManager setup specific version of webdriver in configuration for Selenium

# usage:
#   WatchDoge.configuration.firefox_version = '0.17.0'
#   WatchDoge::WebdriverManager.new(:firefox)

# todo: shouldn't this class singleton with #setup instead of initializing instance?

require 'net_http_ssl_fix'
require 'webdrivers'

module WatchDoge
  class WebdriverManager
    attr_reader :driver
    attr_reader :install_version
    attr_reader :install_path

    def initialize browser
      @driver =
        case browser
        when :chrome
          Webdrivers::Chromedriver
        when :firefox
          Webdrivers::Geckodriver
        end

      @install_version = WatchDoge.configuration.send("#{browser}_version") || @driver.latest_version.to_s

      @install_path = "#{WatchDoge.configuration.web_drivers_dir}/#{browser}/#{@install_version}/"
      Webdrivers.install_dir = @install_path

      @driver.required_version = @install_version

      @driver.update

      klass = "Selenium::WebDriver::#{browser.to_s.split('_').collect(&:capitalize).join}::Service".constantize
      binary =
        case browser
        when :chrome
          "chromedriver"
        when :firefox
          "geckodriver"
        end

      klass.driver_path = "#{@install_path}/#{binary}"
    end
  end
end
